#!/usr/bin/env ruby
=begin
# Copyright 
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#
# Name :          SectionByCamera  
# Version:        V0.2-2012.06.04-2113
# Author :        pgarmyn
# Type :          Tool
#
# Description :   This tool places the camera in (near) the sectionplan.
#                 The parallel view will section the model (without section cuts active).
#                 But the SHADOWS of the entire model will be displayed (if section cuts are switched off).
#                 This will produce a plan or section view with realistic shadow information.
#                 The camera-position is saved in a new scene called "SectionByCamera"
#                 The generated camera should also work with most renderers :
#                       - Works with Twilight.
#                       - Works with Kerkythea, with Twilight exporter
#                       - Doesn't work with SU2KT (from Tomasz, The camera is moved, outside the model).
#
# Usage :         1.	Install into the plugins directory (unzip first)
#                 2.	- Select a section plan
#                       - in the context menu : select 'Section by camera' > '.... automatic'
#                                                   or 'Section by camera' > '.... define camera distance'
#                       - switch off the 'section planes' and 'section cuts' (Menu : View >> Section...)  
#
# Known issues:   The image rendered on the screen is cutted some centimeters before the camera.
#                 The distance between this 'cut' and the camera depends of the dimensions of the model.                
#                 The tools estimates this distance (in '.... automatic'; distance proposed in
#                   '.... define camera distance'), if necessary, it can be adapted in the second option.
#                 If the applied value is to small, the SectionPlane won't be visible after activating 
#                   the tool. This is also true for faces generated by plugins like SectionCutFace (TIG)
#                   or realSection (GWydouw).
#                 SOLUTION : apply a greater distance in '.... define camera distance'
#
# History:        First version V0.1-2012.05.06-1938
#                 2nd version V0.2-2012.06.04-2059   :
#                       Correction for certain non horizontal sections 
# 
=end

require 'sketchup.rb'



def sectionByCam
    
    
    def place_camera(ask_distance=false)
        model = Sketchup.active_model
        sml=0.001       
        ##model.start_operation "SectionByCamera",true,false,false ## only if somthing s created
        nUnit = Sketchup.active_model.options["UnitsOptions"]["LengthUnit"]
        unit = (["inch", "feet","mm","cm","m"])[nUnit]
        prompt = "Distance camera <-> sectionplane :(default unit = "+unit+") : "
        
        sectionplan = model.selection[0]
        bnds = model.bounds
        plane = sectionplan.get_plane
        pcam = bnds.center.project_to_plane plane
        vcam = Geom::Vector3d.new (plane[0],plane[1],plane[2])
        vcam.normalize!
        v_up = [0,0,1]
        if vcam[2]>-0.5 and vcam[2]<0.5
            v_up = [vcam[0],vcam[1],1]
        else
            v_up = [vcam[0],vcam[1],0]
        end        
        v_up = [0,1,0] if v_up[0].abs<sml and v_up[1].abs<sml 
        v_up = [0,1,0] if vcam[0].abs<sml and vcam[1].abs<sml
        v_up = [0,0,1] if vcam[1].abs<sml and vcam[2].abs<sml
       
        puts "vcam "+vcam.inspect
        puts "v_up "+v_up.inspect
        
        maxdim = [bnds.height,bnds.width,bnds.depth].max
        vcam.length = maxdim*5
        pcam_correct = vcam.reverse
        cdistance = (maxdim/500.0).round.to_l
        if ask_distance           
            input = UI.inputbox ([prompt],[cdistance.to_s], " S e c t i o n B y C a m e r a")
            cdistance = input[0].to_l          
        end
        ##pcam_correct.length = vcam.length/500.0
        pcam_correct.length = cdistance
        pcam.offset! pcam_correct
        puts "Camera placed at "+ pcam_correct.length.to_l.to_s##+ unit
        puts "Switch off 'Section cut' and 'Section plan', switch on 'Shadows'"
        ptarget = pcam.offset vcam
        ##model.entities.add_cline(pcam,ptarget)  
        new_cam = Sketchup::Camera.new pcam, ptarget, v_up
        new_cam.perspective = false
        new_cam.aspect_ratio= 0.0
        new_cam.height = maxdim*1.2 
        model.active_view.camera = new_cam
        puts new_cam.direction.inspect
        puts new_cam.up.inspect
        ##model.active_view.zoom sectionplan ## doesnt work, moves camera 
        
        renderingoptions = model.rendering_options
        model.shadow_info["DisplayShadows"]= true
        pages = model.pages
        page = pages.add "SectByCam"
    end
    def validate
        return true if Sketchup.active_model.selection[0].class == Sketchup::SectionPlane
        return false
    end
    #def camerainfo
    #    puts "CamInfo"
    #    cam = Sketchup.active_model.active_view.camera
    #    puts cam.direction.inspect
    #    puts cam.up.inspect
    #end
    
end



if( not file_loaded?("SectionByCam.rb") )
	UI.add_context_menu_handler do |menu|
		if sectionByCam::validate
                        menu.add_separator
                        submenu = menu.add_submenu("Section by camera")
			submenu.add_item("Section by camera : automatic") { sectionByCam::place_camera }
			submenu.add_item("Section by camera : define camera distance") { sectionByCam::place_camera true }
			#submenu.add_item("Caminfo") { sectionByCam::camerainfo }
		end
	end
end
file_loaded("SectionByCam.rb")